(*
  Quido - zaobalen funkc knihovny QuidoStackDll.dll
  Vytvoeno pro Delphi 7

  Vydavatel :  Papouch s.r.o. - http://www.papouch.com/
  Autor     :  Martin Zmenk zamecnik@papouch.com

  popis     : objekt umouje komunikaci se zazenmi Quido pes libovoln rozhran.
              umouje pes spinel protokol zkladn nastaven a ovldn vstup a vstup Quida.
              zaobaluje funkce knihovny QuidoStackDll do objektu.

  Poznmky  : k pouit jsou poteba knihovny CommIntfDll.dll, QuidoStackDll.dll a unita mapujc funkce q_QuidoDllMapD.pas
              v ppad pouit pro USB Quida jsou poteba t nainstalovan ovladae FTD direct drivers a knihova FTD2XX.dll, nebo nainstalovan virtuln sriov port

*)
unit uo_Quido;

interface
uses  q_QuidoDllMapD, classes;
type
  TIOChangeEvent = procedure(Sender:TObject;IOIndex:integer;OldState:boolean;NewState:boolean) of object;
  TQuidoNotifyEvent = procedure(Sender:TObject;Event:word;Param:word) of object;
  TQuido = class
    private
      qHandle:THandle;
      FOnOutputChange: TIOChangeEvent;
      FOnInputChange: TIOChangeEvent;
      FOnInputsChange: TNotifyEvent;
      FOnOutputsChange: TNotifyEvent;
      FOnClose: TNotifyEvent;
      FOnOpen: TNotifyEvent;
      FOnQuidoEvent: TQuidoNotifyEvent;
      procedure SetOnClose(const Value: TNotifyEvent);
      procedure SetOnInputChange(const Value: TIOChangeEvent);
      procedure SetOnInputsChange(const Value: TNotifyEvent);
      procedure SetOnOpen(const Value: TNotifyEvent);
      procedure SetOnOutputChange(const Value: TIOChangeEvent);
      procedure SetOnOutputsChange(const Value: TNotifyEvent);

      function GetInutState(index: integer): boolean;
      function GetOutputState(index: integer): boolean;
      function GetRecentInputState(index: integer): boolean;
      function GetRecentOutputState(index: integer): boolean;
      function GetAutoInputNotification:boolean;
      procedure SetAutoInputNotification(const Value: boolean);
      procedure SetOutputState(index: integer; const Value: boolean);
      procedure SetOnQuidoEvent(const Value: TQuidoNotifyEvent);
      function GetActive: boolean;
      procedure SetActive(const Value: boolean);
      function GetRecentCounterValue(index: integer): cardinal;
    protected
      procedure OnEvent(qHandle:THandle;Event:word;Param:word);
    public //---------------
      constructor Create(ConnectionString:string;QuidoParams:string='ADDR=$FE');overload; //-- vytvoen: connectionString=parametry komunikanho spojen;QuidoParams=parametry quida (adresa)
      class function CreateByDlg(QuidoParams:string='ADDR=$FE';DefaultConnStr:string=''):TQuido; //-- vytvoen pomoc dialogu: connection string se vytvo pomoc dialogu. pokud se v dialogu stiskne storno, nvratov hodnota je NIL.
      destructor Destroy;override;
      procedure Open; //-- oteve spojen s quidem
      procedure Close; //-- zave spojen s quidem

      function GetOutputPWM(Output:Integer; var PWM:Integer):boolean;   // pouze pro zakazkove upravy (custom firmware only)
      function SetOutputPWM(Output:Integer; const PWM:Integer):boolean; // pouze pro zakazkove upravy (custom firmware only)
      function GetInputPPS(Input:Integer; Sec:Integer; var PPS:Integer):boolean; // pouze pro zakazkove upravy (custom firmware only)

      property Active:boolean read GetActive write SetActive; //-- Oteveno nebo zaveno spojen
      function DeviceInfo:string; //-- Informan zprva quida
      function InputsCount:integer; //-- Poet vstup
      function OutputsCount:integer; //-- Poet vstup
      function InputCountersCount:integer; //-- Poet ta vstup
      property InputState[index:integer]:boolean read GetInutState; //-- stavy jednotlivch vstup (te pmo ze zazen)
      property OutputState[index:integer]:boolean read GetOutputState write SetOutputState; //-- zjitn i nastaven jednotlivch vstup (te i zapisuje pmo z/do zazen)
      procedure SetOutputStateTimed(index:integer;State:boolean;TimeSec:integer=0); //-- Nastaven vstupu na urit asov interval (0 = bez asovho intervalu)
      property RecentInputState[index:integer]:boolean read GetRecentInputState; //-- Posledn znm stav vstup - (nezjiuje pmo ze zazen)
                //-- Tyto hodnoty se aktualizuj pomoc metod RefreshInputs, RefreshCecentStates, pouitm vlastnosti InputState, nebo pijetm automaticky vyslan instrukce
      property RecentOutputState[index:integer]:boolean read GetRecentOutputState; //-- Posledn znm stav vstup
                //-- Tyto hodnoty se aktualizuj pomoc metod refreshOutputs,RefreshRecentStates nebo pouttm vlastnosti OutputState
      property RecentCounterValue[index:integer]:cardinal read GetRecentCounterValue; //-- Posledn znm hodnota ta vstup
      procedure GetRecentCounterSetting(index:integer;var Rising,Falling:boolean); //-- Posledn znm nastaven ta
      function GetInputCounterValue(index: integer;NullCounter:boolean=false): cardinal; //-- Zjist hodnotu tae vstupu, monost nslednho vynulovn
      procedure RefreshRecentStates(Inputs:boolean = true;Outputs:boolean=true;Counters:boolean=true;CounterSettings:boolean=true);
                //-- Nate aktuln hodnoty stav vstup, vstup, ta a jejich nastaven.
      procedure RefreshInputs(Wait:boolean=true); //-- Nate aktuln hodnoty vstup ze zazen
      procedure RefreshOutputs(Wait:boolean=true); //-- Nate aktuln hodnoty vstup ze zazen
      property AutoInputNotification:boolean read GetAutoInputNotification write SetAutoInputNotification;
               //-- Nastav i zjist stav automatickho vysln zmny vstup
      property Handle:THandle read qHandle; //-- Originln handle z knihovny QuidoStackDll.dll
      procedure GetCounterSetting(Index:integer;var Raising,Falling:boolean); //-- Zjist nastaven tae vstupu
      procedure SetCounterSetting(Index:integer;Raising,Falling:boolean);  //-- Nastav ta vstupu
      procedure Reset;
      function SndRcvSpin97(Inst:byte;const Data:string;var RcvAck:byte;var RcvData:string;MaxTimeOut:integer=500):boolean;
    published
      property OnQuidoEvent:TQuidoNotifyEvent read FOnQuidoEvent write SetOnQuidoEvent; //-- Udlost - jakkoliv udlost na quidu
      property OnOpen:TNotifyEvent read FOnOpen write SetOnOpen;  //-- Udlost - oteven komunikanho kanlu
      property OnClose:TNotifyEvent read FOnClose write SetOnClose; //-- Udlost - zaven komunikanho kanlu
      property OnInputChange:TIOChangeEvent read FOnInputChange write SetOnInputChange; //-- Udlost - Zmna vstupu
      property OnOutputChange:TIOChangeEvent read FOnOutputChange write SetOnOutputChange; //-- Udlost - Zmna vstup
      property OnInputsChange:TNotifyEvent read FOnInputsChange write SetOnInputsChange; //-- Udlost - Zmna vstupu.
                                //Pozor, vstupy neumouj automatick vysln, tato udlost se tedy vyvol pouze pi zjitn zmn pi explicitnm zjiovn stavu
      property OnOutputsChange:TNotifyEvent read FOnOutputsChange write SetOnOutputsChange; //-- Udlost - Zmna vstp

      class function ConnectionStringDlg(var ConnectionString:string;Caption:string=''):boolean; //-- Metoda tdy, kter vyvol dialog pro vytvoen pipojovacho etzce a vrt ho v parametru. vrac true, pokud bylo stisknuto OK

  end;

implementation
uses sysutils, dialogs;


procedure QuidoCallBackEvent(qHandle:THandle;Event:word;Param:word;CallBackID:cardinal);stdcall;
var EventDone:boolean;
begin
 EventDone:=false;
 try
  if qHandle<>0 then
    if CallBackID<>0 then
      if TObject(CallBackID) is TObject then
        if TObject(CallBackID) is TQuido then
          if TQuido(CallBackID).qHandle = qHandle then
          begin
            EventDone:=true;
            TQuido(CallBackID).OnEvent(qHandle,Event,Param);
          end;
 except
   on e:exception do if EventDone then MessageDlg(e.Message,mtError,[mbOK],0);
 end;
end;

{ TQuido }

procedure TQuido.Close;
begin
 if qClose(qHandle)<>0 then RaiseLastError;
end;

class function TQuido.ConnectionStringDlg(var ConnectionString: string;
  Caption: string): boolean;
begin
  if Caption='' then Caption:='Quido connection string';
  _GetConnectionStringDlg(ConnectionString,Caption);
end;

constructor TQuido.Create(ConnectionString, QuidoParams: string);
begin
  QDTestInit;
  qHandle := qCreateQuido(pansichar(ConnectionString),pansichar(QuidoParams));
  if qHandle = 0 then
  begin
   RaiseLastError;
   raise Exception.Create('Creating Quido failed.');
  end;
  if qRegisterCallBackNotifications(qHandle,QuidoCallBackEvent,cardinal(self))<>0 then RaiseLastError;
end;

class function TQuido.CreateByDlg(QuidoParams: string;DefaultConnStr:string): TQuido;
var s:string;
begin
  S:=DefaultConnStr;
  if _GetConnectionStringDlg(s) then result:=Create(s,QuidoParams) else result:=nil;
end;

destructor TQuido.Destroy;
begin
  qReleaseQuido(qHandle);
  inherited;
end;

function TQuido.DeviceInfo: string;
var i:integer;
begin
  result:='';
  if qInfo(qHandle,nil,i)<>0 then RaiseLastError;
  setlength(result,i);
  if i>0 then
  if qInfo(qHandle,@result[1],i)=0 then result:=copy(result,1,i) else RaiseLastError;
end;

function TQuido.GetActive: boolean;
var B:LongBool;
begin
  if qIsOpened(qHandle,B)<>0 then RaiseLastError;
  result:=B;
end;

function TQuido.GetAutoInputNotification: boolean;
var B:LongBool;
begin
  if qGetAutoNotify(qHandle,B)<>0 then RaiseLastError;
  result:=B;
end;

procedure TQuido.GetCounterSetting(Index: integer; var Raising,
  Falling: boolean);
var R,F:longbool;
begin
  if qGetCounterSetting(qHandle,Index,R,F)<>0 then RaiseLastError
  else begin
    Raising:=R;
    Falling:=F;
  end;
end;

function TQuido.GetInputCounterValue(index: integer;
  NullCounter: boolean): cardinal;
var R:Integer;
begin
  R:=qInputCounterValueByIndex(qHandle,index,result,NullCounter);
  if R<>0 then
  begin
    RaiseLastError;
  end;
end;

function TQuido.GetInputPPS(Input, Sec: Integer;
  var PPS: Integer): boolean;
begin
  result:=false;
  if qGetInputPPS(qHandle,Input,Sec,PPS)<>0
   then RaiseLastError
   else result:=true;
end;

function TQuido.GetInutState(index: integer): boolean;
var B:LongBool;
begin
  if qInputStateByIndex(qHandle,index,B)<>0 then RaiseLastError;
  result:=B;
end;

function TQuido.GetOutputPWM(Output: Integer; var PWM: Integer): boolean;
begin
  result:=false;
  if qGetOutputPWM(qHandle,Output,PWM)<>0
   then RaiseLastError
   else result:=true;
end;

function TQuido.GetOutputState(index: integer): boolean;
var B:LongBool;
begin
  if qOutputStateByIndex(qHandle,index,B)<>0 then RaiseLastError;
  result:=B;
end;

procedure TQuido.GetRecentCounterSetting(index: integer; var Rising,
  Falling:boolean);
  var R,F:longbool;
begin
  if qGetRecentCounterSettingByIndex(qHandle,Index,R,F)<>0 then RaiseLastError;
  Rising:=R;
  Falling:=F;
end;

function TQuido.GetRecentCounterValue(index: integer): cardinal;
var B:integer;
begin
  if qGetRecentInputCounterByIndex(qHandle,index,B)<>0 then RaiseLastError;
  result:=B;
end;

function TQuido.GetRecentInputState(index: integer): boolean;
var B:LongBool;
begin
  if qGetRecentInputByIndex(qHandle,index,B)<>0 then RaiseLastError;
  result:=B;
end;

function TQuido.GetRecentOutputState(index: integer): boolean;
var B:LongBool;
begin
  if qGetRecentOutputByIndex(qHandle,index,B)<>0 then RaiseLastError;
  result:=B;
end;

function TQuido.InputCountersCount: integer;
begin
  result:=qInputCountersCount(qHandle);
  if result<0 then RaiseLastError;
end;

function TQuido.InputsCount: integer;
begin
  result:=qInputsCount(qHandle);
  if result<0 then RaiseLastError;
end;

procedure TQuido.OnEvent(qHandle: THandle; Event, Param: word);
begin
  if qHandle=self.qHandle then
  begin
    if assigned(fOnQuidoEvent) then FOnQuidoEvent(self,event,param);
    case Event of
      qev_CIOpen : if assigned(fOnOpen) then FOnOpen(self);
      qev_CIClose : if assigned(fOnClose) then FOnClose(self);
      qev_InputsChange : if assigned(fOnInputsChange) then fOnInputsChange(self);
      qev_OutputsChange : if assigned(fOnOutputsChange) then fOnOutputsChange(self);
      qev_InputChange : if assigned(fOnInputChange) then fOnInputChange(self, Param and $FF, ((Param shr 8)and 2)=2, ((Param shr 8)and 1)=1);
      qev_OutputChange : if assigned(fOnOutputChange) then fOnOutputChange(self, Param and $FF, ((Param shr 8)and 2)=2, ((Param shr 8)and 1)=1);
    end;
  end;
end;

procedure TQuido.Open;
begin
 if qOpen(qHandle)<>0 then RaiseLastError;
end;

function TQuido.OutputsCount: integer;
begin
 result:=qOutputsCount(qHandle);
 if result<0 then RaiseLastError;
end;

procedure TQuido.RefreshInputs(Wait:boolean);
begin
 if qRefreshRecentStates(qHandle,true,false,false,false)<>0 then RaiseLastError;
end;

procedure TQuido.RefreshOutputs(Wait:boolean);
begin
 if qRefreshRecentStates(qHandle,false,true,false,false)<>0 then RaiseLastError;
end;

procedure TQuido.RefreshRecentStates(Inputs, Outputs, Counters,
  CounterSettings: boolean);
begin
  if qRefreshRecentStates(qHandle,Inputs,Outputs,Counters,CounterSettings)<>0 then RaiseLastError;
end;

procedure TQuido.Reset;
begin
  if qReset(qHandle)<>0 then RaiseLastError;
end;

procedure TQuido.SetActive(const Value: boolean);
var B:boolean;
    ret:integer;
begin
  B:=GetActive;
  ret:=0;
  if B<>Value then
    if Value then ret:=qOpen(qHandle) else ret:=qClose(qHandle);
  if ret<>0 then RaiseLastError;
end;

procedure TQuido.SetAutoInputNotification(const Value: boolean);
var B:LongBool;
begin
  B:=Value;
  if qSetAutoNotify(qHandle,B)<>0 then RaiseLastError;
end;

procedure TQuido.SetCounterSetting(Index: integer; Raising,
  Falling: boolean);
begin
  if qSetCounterSetting(qHandle,Index,Raising,Falling)<>0 then RaiseLastError;
end;

procedure TQuido.SetOnClose(const Value: TNotifyEvent);
begin
  FOnClose := Value;
end;

procedure TQuido.SetOnInputChange(const Value: TIOChangeEvent);
begin
  FOnInputChange := Value;
end;

procedure TQuido.SetOnInputsChange(const Value: TNotifyEvent);
begin
  FOnInputsChange := Value;
end;

procedure TQuido.SetOnOpen(const Value: TNotifyEvent);
begin
  FOnOpen := Value;
end;

procedure TQuido.SetOnOutputChange(const Value: TIOChangeEvent);
begin
  FOnOutputChange := Value;
end;

procedure TQuido.SetOnOutputsChange(const Value: TNotifyEvent);
begin
  FOnOutputsChange := Value;
end;

procedure TQuido.SetOnQuidoEvent(const Value: TQuidoNotifyEvent);
begin
  FOnQuidoEvent := Value;
end;

function TQuido.SetOutputPWM(Output: Integer; const PWM: Integer): boolean;
begin
  result:=false;
  if qSetOutputPWM(qHandle,Output,PWM)<>0
   then RaiseLastError
   else result:=true;
end;

procedure TQuido.SetOutputState(index: integer; const Value: boolean);
begin
 if qSetOutput(qHandle,index,Value)<>0 then RaiseLastError;
end;

procedure TQuido.SetOutputStateTimed(index: integer; State: boolean;
  TimeSec: integer);
begin
 if qSetOutputTimered(qHandle,index,State,TimeSec)<>0 then RaiseLastError;
end;

function TQuido.SndRcvSpin97(Inst: byte; const Data: string;
  var RcvAck: byte;var RcvData: string; MaxTimeOut: integer): boolean;
begin
  result:=_SndRcvSpin97(qHandle,Inst,Data,RcvAck,RcvData,MaxTimeOut);
end;

end.
